"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.connectable = void 0;
const Subject_1 = require("../Subject");
const Observable_1 = require("../Observable");
const defer_1 = require("./defer");
const DEFAULT_CONFIG = {
    connector: () => new Subject_1.Subject(),
    resetOnDisconnect: true,
};
function connectable(source, config = DEFAULT_CONFIG) {
    let connection = null;
    const { connector, resetOnDisconnect = true } = config;
    let subject = connector();
    const result = new Observable_1.Observable((subscriber) => {
        return subject.subscribe(subscriber);
    });
    result.connect = () => {
        if (!connection || connection.closed) {
            connection = (0, defer_1.defer)(() => source).subscribe(subject);
            if (resetOnDisconnect) {
                connection.add(() => (subject = connector()));
            }
        }
        return connection;
    };
    return result;
}
exports.connectable = connectable;
//# sourceMappingURL=connectable.js.map