"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onErrorResumeNext = void 0;
const Observable_1 = require("../Observable");
const argsOrArgArray_1 = require("../util/argsOrArgArray");
const Subscriber_1 = require("../Subscriber");
const noop_1 = require("../util/noop");
const from_1 = require("./from");
function onErrorResumeNext(...sources) {
    const nextSources = (0, argsOrArgArray_1.argsOrArgArray)(sources);
    return new Observable_1.Observable((destination) => {
        let sourceIndex = 0;
        const subscribeNext = () => {
            if (sourceIndex < nextSources.length) {
                let nextSource;
                try {
                    nextSource = (0, from_1.from)(nextSources[sourceIndex++]);
                }
                catch (err) {
                    subscribeNext();
                    return;
                }
                const innerSubscriber = (0, Subscriber_1.operate)({ destination, error: noop_1.noop, complete: noop_1.noop });
                nextSource.subscribe(innerSubscriber);
                innerSubscriber.add(subscribeNext);
            }
            else {
                destination.complete();
            }
        };
        subscribeNext();
    });
}
exports.onErrorResumeNext = onErrorResumeNext;
//# sourceMappingURL=onErrorResumeNext.js.map