"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bufferCount = void 0;
const Observable_1 = require("../Observable");
const Subscriber_1 = require("../Subscriber");
const arrRemove_1 = require("../util/arrRemove");
function bufferCount(bufferSize, startBufferEvery = null) {
    startBufferEvery = startBufferEvery ?? bufferSize;
    return (source) => new Observable_1.Observable((destination) => {
        let buffers = [];
        let count = 0;
        source.subscribe((0, Subscriber_1.operate)({
            destination,
            next: (value) => {
                let toEmit = null;
                if (count++ % startBufferEvery === 0) {
                    buffers.push([]);
                }
                for (const buffer of buffers) {
                    buffer.push(value);
                    if (bufferSize <= buffer.length) {
                        toEmit = toEmit ?? [];
                        toEmit.push(buffer);
                    }
                }
                if (toEmit) {
                    for (const buffer of toEmit) {
                        (0, arrRemove_1.arrRemove)(buffers, buffer);
                        destination.next(buffer);
                    }
                }
            },
            complete: () => {
                for (const buffer of buffers) {
                    destination.next(buffer);
                }
                destination.complete();
            },
            finalize: () => {
                buffers = null;
            },
        }));
    });
}
exports.bufferCount = bufferCount;
//# sourceMappingURL=bufferCount.js.map