"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.debounce = void 0;
const Subscriber_1 = require("../Subscriber");
const Observable_1 = require("../Observable");
const noop_1 = require("../util/noop");
const from_1 = require("../observable/from");
function debounce(durationSelector) {
    return (source) => new Observable_1.Observable((destination) => {
        let hasValue = false;
        let lastValue = null;
        let durationSubscriber = null;
        const emit = () => {
            durationSubscriber?.unsubscribe();
            durationSubscriber = null;
            if (hasValue) {
                hasValue = false;
                const value = lastValue;
                lastValue = null;
                destination.next(value);
            }
        };
        source.subscribe((0, Subscriber_1.operate)({
            destination,
            next: (value) => {
                durationSubscriber?.unsubscribe();
                hasValue = true;
                lastValue = value;
                durationSubscriber = (0, Subscriber_1.operate)({ destination, next: emit, complete: noop_1.noop });
                (0, from_1.from)(durationSelector(value)).subscribe(durationSubscriber);
            },
            complete: () => {
                emit();
                destination.complete();
            },
            finalize: () => {
                lastValue = durationSubscriber = null;
            },
        }));
    });
}
exports.debounce = debounce;
//# sourceMappingURL=debounce.js.map