"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.debounceTime = void 0;
const async_1 = require("../scheduler/async");
const Observable_1 = require("../Observable");
const Subscriber_1 = require("../Subscriber");
function debounceTime(dueTime, scheduler = async_1.asyncScheduler) {
    return (source) => new Observable_1.Observable((destination) => {
        let activeTask = null;
        let lastValue = null;
        let lastTime = null;
        let scheduling = false;
        const emit = () => {
            if (scheduling || activeTask) {
                if (activeTask) {
                    activeTask.unsubscribe();
                    activeTask = null;
                }
                const value = lastValue;
                lastValue = null;
                destination.next(value);
            }
        };
        function emitWhenIdle() {
            const targetTime = lastTime + dueTime;
            const now = scheduler.now();
            if (now < targetTime) {
                activeTask = this.schedule(undefined, targetTime - now);
                destination.add(activeTask);
                return;
            }
            emit();
        }
        source.subscribe((0, Subscriber_1.operate)({
            destination,
            next: (value) => {
                lastValue = value;
                lastTime = scheduler.now();
                if (!activeTask) {
                    scheduling = true;
                    activeTask = scheduler.schedule(emitWhenIdle, dueTime);
                    scheduling = false;
                    destination.add(activeTask);
                }
            },
            complete: () => {
                emit();
                destination.complete();
            },
            finalize: () => {
                lastValue = activeTask = null;
            },
        }));
    });
}
exports.debounceTime = debounceTime;
//# sourceMappingURL=debounceTime.js.map