"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.distinct = void 0;
const Observable_1 = require("../Observable");
const Subscriber_1 = require("../Subscriber");
const noop_1 = require("../util/noop");
const from_1 = require("../observable/from");
function distinct(keySelector, flushes) {
    return (source) => new Observable_1.Observable((destination) => {
        const distinctKeys = new Set();
        source.subscribe((0, Subscriber_1.operate)({
            destination,
            next: (value) => {
                const key = keySelector ? keySelector(value) : value;
                if (!distinctKeys.has(key)) {
                    distinctKeys.add(key);
                    destination.next(value);
                }
            },
        }));
        flushes && (0, from_1.from)(flushes).subscribe((0, Subscriber_1.operate)({ destination, next: () => distinctKeys.clear(), complete: noop_1.noop }));
    });
}
exports.distinct = distinct;
//# sourceMappingURL=distinct.js.map