"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.repeat = void 0;
const empty_1 = require("../observable/empty");
const Observable_1 = require("../Observable");
const Subscriber_1 = require("../Subscriber");
const from_1 = require("../observable/from");
const timer_1 = require("../observable/timer");
function repeat(countOrConfig) {
    let count = Infinity;
    let delay;
    if (countOrConfig != null) {
        if (typeof countOrConfig === 'object') {
            ({ count = Infinity, delay } = countOrConfig);
        }
        else {
            count = countOrConfig;
        }
    }
    return count <= 0
        ? () => empty_1.EMPTY
        : (source) => new Observable_1.Observable((destination) => {
            let soFar = 0;
            let sourceSub;
            const resubscribe = () => {
                sourceSub?.unsubscribe();
                sourceSub = null;
                if (delay != null) {
                    const notifier = typeof delay === 'number' ? (0, timer_1.timer)(delay) : (0, from_1.from)(delay(soFar));
                    const notifierSubscriber = (0, Subscriber_1.operate)({
                        destination,
                        next: () => {
                            notifierSubscriber.unsubscribe();
                            subscribeToSource();
                        },
                    });
                    notifier.subscribe(notifierSubscriber);
                }
                else {
                    subscribeToSource();
                }
            };
            const subscribeToSource = () => {
                let syncUnsub = false;
                sourceSub = source.subscribe((0, Subscriber_1.operate)({
                    destination,
                    complete: () => {
                        if (++soFar < count) {
                            if (sourceSub) {
                                resubscribe();
                            }
                            else {
                                syncUnsub = true;
                            }
                        }
                        else {
                            destination.complete();
                        }
                    },
                }));
                if (syncUnsub) {
                    resubscribe();
                }
            };
            subscribeToSource();
        });
}
exports.repeat = repeat;
//# sourceMappingURL=repeat.js.map