"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.skipLast = void 0;
const identity_1 = require("../util/identity");
const Observable_1 = require("../Observable");
const Subscriber_1 = require("../Subscriber");
function skipLast(skipCount) {
    return skipCount <= 0
        ?
            identity_1.identity
        : (source) => new Observable_1.Observable((destination) => {
            let ring = new Array(skipCount);
            let seen = 0;
            source.subscribe((0, Subscriber_1.operate)({
                destination,
                next: (value) => {
                    const valueIndex = seen++;
                    if (valueIndex < skipCount) {
                        ring[valueIndex] = value;
                    }
                    else {
                        const index = valueIndex % skipCount;
                        const oldValue = ring[index];
                        ring[index] = value;
                        destination.next(oldValue);
                    }
                },
            }));
            return () => {
                ring = null;
            };
        });
}
exports.skipLast = skipLast;
//# sourceMappingURL=skipLast.js.map