"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.takeWhile = void 0;
const Observable_1 = require("../Observable");
const Subscriber_1 = require("../Subscriber");
function takeWhile(predicate, inclusive = false) {
    return (source) => new Observable_1.Observable((destination) => {
        let index = 0;
        const operatorSubscriber = (0, Subscriber_1.operate)({
            destination,
            next: (value) => {
                if (predicate(value, index++)) {
                    destination.next(value);
                }
                else {
                    operatorSubscriber.unsubscribe();
                    if (inclusive) {
                        destination.next(value);
                    }
                    destination.complete();
                }
            },
        });
        source.subscribe(operatorSubscriber);
    });
}
exports.takeWhile = takeWhile;
//# sourceMappingURL=takeWhile.js.map