"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.window = void 0;
const Observable_1 = require("../Observable");
const Subject_1 = require("../Subject");
const Subscriber_1 = require("../Subscriber");
const noop_1 = require("../util/noop");
const from_1 = require("../observable/from");
function window(windowBoundaries) {
    return (source) => new Observable_1.Observable((destination) => {
        let windowSubject = new Subject_1.Subject();
        destination.next(windowSubject.asObservable());
        const errorHandler = (err) => {
            windowSubject.error(err);
            destination.error(err);
        };
        source.subscribe((0, Subscriber_1.operate)({
            destination,
            next: (value) => windowSubject?.next(value),
            complete: () => {
                windowSubject.complete();
                destination.complete();
            },
            error: errorHandler,
        }));
        (0, from_1.from)(windowBoundaries).subscribe((0, Subscriber_1.operate)({
            destination,
            next: () => {
                windowSubject.complete();
                destination.next((windowSubject = new Subject_1.Subject()));
            },
            complete: noop_1.noop,
            error: errorHandler,
        }));
        return () => {
            windowSubject?.unsubscribe();
            windowSubject = null;
        };
    });
}
exports.window = window;
//# sourceMappingURL=window.js.map