import { Observable } from '../Observable';
import { Subject } from '../Subject';
import { operate } from '../Subscriber';
import { noop } from '../util/noop';
import { from } from '../observable/from';
export function window(windowBoundaries) {
    return (source) => new Observable((destination) => {
        let windowSubject = new Subject();
        destination.next(windowSubject.asObservable());
        const errorHandler = (err) => {
            windowSubject.error(err);
            destination.error(err);
        };
        source.subscribe(operate({
            destination,
            next: (value) => windowSubject?.next(value),
            complete: () => {
                windowSubject.complete();
                destination.complete();
            },
            error: errorHandler,
        }));
        from(windowBoundaries).subscribe(operate({
            destination,
            next: () => {
                windowSubject.complete();
                destination.next((windowSubject = new Subject()));
            },
            complete: noop,
            error: errorHandler,
        }));
        return () => {
            windowSubject?.unsubscribe();
            windowSubject = null;
        };
    });
}
//# sourceMappingURL=window.js.map